//
//  CurrentInputController.m
//  CurrentInput
//
//  Created by Phidgets on 2015-07-30.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "SoundSensorController.h"

@implementation SoundSensorController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetSoundSensor_delete(&ch);
        ch = nil;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetSoundSensor_create(&ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetSoundSensor_setOnSPLChangeHandler(ch, gotSPLChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    if(!octaves)
        octaves = [[NSMutableArray alloc] init];
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
    
    // Create graph
    CPTXYGraph *graph = [[CPTXYGraph alloc] initWithFrame:NSRectToCGRect(graphView.bounds)];
    graph.fill                          = [CPTFill fillWithColor:[CPTColor clearColor]];
    graph.cornerRadius                  = 20.0;
    graph.plotAreaFrame.masksToBorder   = NO;
    graph.paddingLeft                   = 50.0f;
    graph.paddingTop                    = 30.0f;
    graph.paddingRight                  = 30.0f;
    graph.paddingBottom                 = 30.0f;
    graphView.hostedGraph = graph;
    
    
    // Add plot space for horizontal bar charts
    CPTXYPlotSpace *barPlotSpace = (CPTXYPlotSpace *)[graph defaultPlotSpace];
    barPlotSpace.xRange = [CPTPlotRange plotRangeWithLocation:@0.0 length:@10.0];
    barPlotSpace.yRange = [CPTPlotRange plotRangeWithLocation:@0.0 length:@102.0];
    [graph addPlotSpace:barPlotSpace];
    
    CPTMutableLineStyle *barLineStyle = [[CPTMutableLineStyle alloc] init];
    [barLineStyle setLineColor:[CPTColor blackColor]];
    [barLineStyle setLineWidth:1.0];
    
    // First bar plot
    barPlot = [[CPTBarPlot alloc] init];
    barPlot.fill = [CPTFill fillWithColor:[CPTColor redColor]];
    barPlot.barWidth          = @1.0;
    barPlot.barCornerRadius   = 4.0;
    barPlot.barsAreHorizontal = NO;
    barPlot.dataSource        = self;
    barPlot.delegate          = self;
    barPlot.lineStyle         = barLineStyle;
    barPlot.identifier        = @"Octaves";
    
    [graph addPlot:barPlot toPlotSpace:barPlotSpace];
    
    //Format axes
    CPTMutableLineStyle *axisLineStyle = [[CPTMutableLineStyle alloc] init];
    [axisLineStyle setLineColor:[CPTColor blackColor]];
    [axisLineStyle setLineWidth:2.0];
    CPTXYAxisSet *axisSet = (CPTXYAxisSet *)graphView.hostedGraph.axisSet;
    //X-Axis
    CPTXYAxis *xAxis = axisSet.xAxis;
    double x = 0.5;
    NSMutableArray *xLabels = [NSMutableArray array];
    NSArray *labels = [NSArray arrayWithObjects:@"32Hz",@"63Hz",@"125Hz",@"250Hz",@"500Hz",@"1kHz",@"2kHz",@"4kHz",@"8kHz",@"16kHz",nil];
    CPTMutableTextStyle *textStyle = [[CPTMutableTextStyle alloc] init];
    [textStyle setFontSize:11];
    for (NSString *label in labels) {
        CPTAxisLabel *newLabel = [[CPTAxisLabel alloc] initWithText:label textStyle:textStyle];
        newLabel.tickLocation   = [NSDecimalNumber numberWithDouble:x++];
        newLabel.offset         = xAxis.labelOffset + xAxis.majorTickLength;
        [xLabels addObject:newLabel];
    }
    xAxis.axisLabels = [NSSet setWithArray:xLabels];
    xAxis.labelingPolicy = CPTAxisLabelingPolicyNone;
    xAxis.axisLineStyle = axisLineStyle;
    //Y-Axis
    NSNumberFormatter *formatter = [[NSNumberFormatter alloc] init];
    formatter.positiveSuffix = @" dB";
    CPTXYAxis *yAxis = axisSet.yAxis;
    yAxis.labelingPolicy = CPTAxisLabelingPolicyAutomatic;
    yAxis.axisLineStyle = axisLineStyle;
    yAxis.labelFormatter = formatter;
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}


#pragma mark Event callbacks

static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

void gotSPLChangeData(PhidgetSoundSensorHandle phid, void *context, double dBZ, double dBA, double dBC, const double octaves[10]){
    NSMutableArray *array = [[NSMutableArray alloc] init];
    NSMutableArray *octaveArray = [[NSMutableArray alloc] init];
    [array addObject:[NSNumber numberWithDouble:dBZ]];
    [array addObject:[NSNumber numberWithDouble:dBA]];
    [array addObject:[NSNumber numberWithDouble:dBC]];
    for(int i = 0; i < 10; i ++){
        [octaveArray addObject:[NSNumber numberWithDouble:octaves[i]]];
        
    }
    [array addObject:octaveArray];
    [(__bridge id)context performSelectorOnMainThread:@selector(onSPLChangeHandler:)
                                           withObject:array
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    Phidget_DeviceID deviceID;
    uint32 minDataInterval, maxDataInterval, dataInterval;
    double minSPLChangeTrigger,maxSPLChangeTrigger, splChangeTrigger;
    PhidgetSoundSensor_SPLRange splRange;
    
    //Get information from channel which will allow us to configure the GUI properly
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetSoundSensor_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetSoundSensor_getMinDataInterval(ch, &minDataInterval);
    PhidgetSoundSensor_getDataInterval(ch, &dataInterval);
    PhidgetSoundSensor_getMaxSPLChangeTrigger(ch, &maxSPLChangeTrigger);
    PhidgetSoundSensor_getMinSPLChangeTrigger(ch, &minSPLChangeTrigger);
    PhidgetSoundSensor_getSPLChangeTrigger(ch, &splChangeTrigger);
    PhidgetSoundSensor_getSPLRange(ch, &splRange);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [settingsBox setHidden:NO];
    [dataBox setHidden:NO];
    
    //Configure data interval slider with information read from channel
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setIntValue:dataInterval];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    
    //Configure trigger slider with information read from channel
    [maxChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f dB",maxSPLChangeTrigger]];
    [minChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f dB",minSPLChangeTrigger]];
    [changeTriggerSlider setMaxValue:maxSPLChangeTrigger];
    [changeTriggerSlider setMinValue:minSPLChangeTrigger];
    [changeTriggerSlider setDoubleValue:splChangeTrigger];
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f dB", splChangeTrigger]];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onSPLChangeHandler:(NSMutableArray *)array{
    double dBZ = [[array objectAtIndex:0] doubleValue];
    double dBA = [[array objectAtIndex:1] doubleValue];
    double dBC = [[array objectAtIndex:2] doubleValue];
    [dBZText setStringValue:[NSString stringWithFormat:@"%.2f",dBZ]];
    [dBAText setStringValue:[NSString stringWithFormat:@"%.2f",dBA]];
    [dBCText setStringValue:[NSString stringWithFormat:@"%.2f",dBC]];
    octaves = [array objectAtIndex:3];
    [barPlot reloadData];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)outputLastError {
	const char *errorString;
	char *errorDetailString = NULL;
    size_t errorDetailStringLen = 0;
    PhidgetReturnCode lastErr;
	
    if (!Phidget_getLastError(&lastErr, &errorString, NULL, &errorDetailStringLen)) {
		errorDetailString = malloc(errorDetailStringLen);
		Phidget_getLastError(&lastErr, &errorString, errorDetailString, &errorDetailStringLen);
    	[self outputError:errorDetailString];
    	free(errorDetailString);
    }
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}


#pragma mark GUI controls
- (IBAction)setChangeTrigger:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetSoundSensor_setSPLChangeTrigger(ch, [changeTriggerSlider doubleValue]);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
    }
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f dB", [changeTriggerSlider doubleValue]]];
}

- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetSoundSensor_setDataInterval(ch, [dataIntervalSlider intValue]);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
    }
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
}

- (IBAction)setSPLRange:(id)sender {
    PhidgetReturnCode result;
    CPTXYPlotSpace *barPlotSpace = (CPTXYPlotSpace *)[graphView.hostedGraph defaultPlotSpace];
    
    if([splRangeCombo indexOfSelectedItem] == 0){
        result = PhidgetSoundSensor_setSPLRange(ch, SPL_RANGE_102dB);
         barPlotSpace.yRange = [CPTPlotRange plotRangeWithLocation:@0.0 length:@102.0];
    }
    else{
        result = PhidgetSoundSensor_setSPLRange(ch, SPL_RANGE_82dB);
        barPlotSpace.yRange = [CPTPlotRange plotRangeWithLocation:@0.0 length:@82.0];
    }

    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
}

#pragma mark core plot
- (NSUInteger)numberOfRecordsForPlot:(nonnull CPTPlot *)plot {
    return 10;
}

-(double)doubleForPlot:(CPTPlot *)plot field:(NSUInteger)fieldEnum recordIndex:(NSUInteger)idx{
    if(fieldEnum == CPTBarPlotFieldBarTip){
    if([octaves count] != 0){
        return [[octaves objectAtIndex:idx] doubleValue];
    }
    else
        return 0.0;
    }
    else{
        return idx + 0.5;
    }
}

@end
